@@ -1,13 +1,12 @@
+from collections import ChainMap
 from datetime import datetime
-import json
 import logging
 import sys
 from ..utils.utils import pretty_print_except
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Dict
 
 from ThermiaOnlineAPI.const import (
-    OPERATIONAL_TIME_REGISTERS,
     REG_BRINE_IN,
     REG_BRINE_OUT,
     REG_ACTUAL_POOL_TEMP,
@@ -25,7 +24,6 @@ from ThermiaOnlineAPI.const import (
     REG_OPER_TIME_IMM3,
     REG_RETURN_LINE,
     REG_SUPPLY_LINE,
-    TEMPERATURE_REGISTERS,
     DATETIME_FORMAT,
 )
 
@@ -34,7 +32,7 @@ from ..utils.utils import get_dict_value_safe
 if TYPE_CHECKING:
     from ..api.ThermiaAPI import ThermiaAPI
 
-DEFAULT_REGISTER_INDEXES = {
+DEFAULT_REGISTER_INDEXES: Dict[str, int | None] = {
     "temperature": None,
     "operation_mode": None,
     "hot_water_switch": None,
@@ -42,7 +40,7 @@ DEFAULT_REGISTER_INDEXES = {
 
 
 class ThermiaHeatPump:
-    def __init__(self, device_data: json, api_interface: "ThermiaAPI"):
+    def __init__(self, device_data: dict, api_interface: "ThermiaAPI"):
         self.__device_id = str(device_data["id"])
         self.__api_interface = api_interface
 
@@ -71,8 +69,8 @@ class ThermiaHeatPump:
         self.__status = self.__api_interface.get_device_status(self.__device_id)
         self.__device_data = self.__api_interface.get_device_by_id(self.__device_id)
 
-        self.__register_indexes["temperature"] = self.__status.get(
-            "heatingEffectRegisters", [None, None]
+        self.__register_indexes["temperature"] = get_dict_value_safe(
+            self.__status, "heatingEffectRegisters", [None, None]
         )[1]
 
         self.__group_temperatures = self.__api_interface.get__group_temperatures(
@@ -101,7 +99,12 @@ class ThermiaHeatPump:
         self.__register_indexes["hot_water_switch"] = register_index
 
     def set_temperature(self, temperature: int):
+        if self.__status is None:
+            self._LOGGER.error("Status not available, cannot set temperature")
+            return
+
         self._LOGGER.info("Setting temperature to " + str(temperature))
+
         self.__status[
             "heatingEffect"
         ] = temperature  # update local state before refetching data
@@ -130,7 +133,13 @@ class ThermiaHeatPump:
         self.update_data()
 
     def get_all_available_register_groups(self):
-        installation_profile_id = self.__info.get("installationProfileId")
+        installation_profile_id = get_dict_value_safe(
+            self.__info, "installationProfileId"
+        )
+
+        if installation_profile_id is None:
+            return []
+
         register_groups = self.__api_interface.get_all_available_groups(
             installation_profile_id
         )
@@ -153,16 +162,16 @@ class ThermiaHeatPump:
     def get_register_data_by_register_group_and_name(
         self, register_group: str, register_name: str
     ):
-        register_group = self.__api_interface.get_register_group_json(
+        register_group_data: list = self.__api_interface.get_register_group_json(
             self.__device_id, register_group
         )
 
-        if register_group is None:
+        if register_group_data is None:
             self._LOGGER.error("No register group found for group: " + register_group)
             return None
 
         return self.__get_data_from_group_by_register_name(
-            register_group, register_name
+            register_group_data, register_name
         )
 
     def set_register_data_by_register_group_and_name(
@@ -211,20 +220,26 @@ class ThermiaHeatPump:
         }
 
     def __get_temperature_data_by_register_name(
-        self, register_name: TEMPERATURE_REGISTERS
+        self, register_name: str  # TEMPERATURE_REGISTERS
     ):
+        if self.__group_temperatures is None:
+            return None
+
         return self.__get_data_from_group_by_register_name(
             self.__group_temperatures, register_name
         )
 
     def __get_operational_time_data_by_register_name(
-        self, register_name: OPERATIONAL_TIME_REGISTERS
+        self, register_name: str  # OPERATIONAL_TIME_REGISTERS
     ):
+        if self.__group_operational_time is None:
+            return None
+
         return self.__get_data_from_group_by_register_name(
             self.__group_operational_time, register_name
         )
 
-    def __get_data_from_group_by_register_name(self, group, register_name: str):
+    def __get_data_from_group_by_register_name(self, group: list, register_name: str):
         if group is None:
             return None
 
@@ -247,7 +262,8 @@ class ThermiaHeatPump:
 
     def __get_active_alarms(self):
         active_alarms = filter(
-            lambda alarm: alarm.get("isActiveAlarm", False) is True, self.__alarms
+            lambda alarm: get_dict_value_safe(alarm, "isActiveAlarm", False) is True,
+            self.__alarms or [],
         )
         return list(active_alarms)
 
@@ -264,9 +280,61 @@ class ThermiaHeatPump:
 
         self.__historical_data_registers_map = data_map
 
+    def __get_register_from_operational_status(self, register_name: str) -> dict | None:
+        data = [
+            d
+            for d in self.__group_operational_status or []
+            if d["registerName"] == register_name
+        ]
+
+        if len(data) != 1:
+            return None
+
+        return data[0]
+
+    def __get_value_by_key_and_register_name_from_operational_status(
+        self, register_name: str, value_key: str
+    ):
+        data_from_register = self.__get_register_from_operational_status(register_name)
+
+        if not data_from_register:
+            return None
+
+        register_values_list = data_from_register.get("valueNames", [])
+        values_list: list = [d for d in register_values_list if d["name"] == value_key]
+
+        if len(values_list) != 1:
+            return None
+
+        value: dict = values_list[0]
+
+        if value.get("visible"):
+            return value.get("value")
+
+        return None
+
+    def __get_all_operational_statuses_from_operational_status(self) -> ChainMap | None:
+        data = self.__get_register_from_operational_status(
+            "REG_OPERATIONAL_STATUS_PRIO1"
+        )
+
+        if not data:
+            return None
+
+        register_values_list = data.get("valueNames", [])
+
+        operation_modes_map = map(
+            lambda values: {
+                values.get("value"): values.get("name").split("REG_VALUE_STATUS_")[1],
+            },
+            register_values_list,
+        )
+        operation_modes_list = list(operation_modes_map)
+        return ChainMap(*operation_modes_list)
+
     @property
     def name(self):
-        return self.__info.get("name")
+        return get_dict_value_safe(self.__info, "name")
 
     @property
     def id(self):
@@ -274,44 +342,44 @@ class ThermiaHeatPump:
 
     @property
     def is_online(self):
-        return self.__info.get("isOnline")
+        return get_dict_value_safe(self.__info, "isOnline")
 
     @property
     def last_online(self):
-        return self.__info.get("lastOnline")
+        return get_dict_value_safe(self.__info, "lastOnline")
 
     @property
     def model(self):
-        return self.__device_data.get("profile", {}).get("thermiaName")
+        return get_dict_value_safe(self.__device_data, "profile", {}).get("thermiaName")
 
     @property
     def has_indoor_temp_sensor(self):
-        return self.__status.get("hasIndoorTempSensor")
+        return get_dict_value_safe(self.__status, "hasIndoorTempSensor")
 
     @property
     def indoor_temperature(self):
         if self.has_indoor_temp_sensor:
-            return self.__status.get("indoorTemperature")
+            return get_dict_value_safe(self.__status, "indoorTemperature")
         else:
             return self.heat_temperature
 
     @property
     def is_outdoor_temp_sensor_functioning(self):
-        return self.__status.get("isOutdoorTempSensorFunctioning")
+        return get_dict_value_safe(self.__status, "isOutdoorTempSensorFunctioning")
 
     @property
     def outdoor_temperature(self):
-        return self.__status.get("outdoorTemperature")
+        return get_dict_value_safe(self.__status, "outdoorTemperature")
 
     @property
     def is_hot_water_active(self):
-        return self.__status.get("isHotwaterActive") or self.__status.get(
-            "isHotWaterActive"
-        )
+        return get_dict_value_safe(
+            self.__status, "isHotwaterActive"
+        ) or get_dict_value_safe(self.__status, "isHotWaterActive")
 
     @property
     def hot_water_temperature(self):
-        return self.__status.get("hotWaterTemperature")
+        return get_dict_value_safe(self.__status, "hotWaterTemperature")
 
     ###########################################################################
     # Heat temperature data
@@ -319,7 +387,7 @@ class ThermiaHeatPump:
 
     @property
     def heat_temperature(self):
-        return self.__status.get("heatingEffect")
+        return get_dict_value_safe(self.__status, "heatingEffect")
 
     @property
     def heat_min_temperature_value(self):
@@ -407,24 +475,117 @@ class ThermiaHeatPump:
         )
 
     ###########################################################################
-    # Operational status data
+    # Operational status (REG_GROUP_OPERATIONAL_STATUS)
     ###########################################################################
 
     @property
     def operational_status(self):
-        return get_dict_value_safe(self.__group_operational_status, "current")
+        data = self.__get_register_from_operational_status(
+            "REG_OPERATIONAL_STATUS_PRIO1"
+        )
+
+        if not data:
+            return None
+
+        current_register_value = get_dict_value_safe(data, "registerValue")
+
+        data = self.__get_all_operational_statuses_from_operational_status()
+
+        if not data:
+            return None
+
+        current_operation_mode = [
+            name for value, name in data.items() if value == current_register_value
+        ]
+
+        if len(current_operation_mode) != 1:
+            return None
+
+        return current_operation_mode[0]
 
     @property
     def available_operational_statuses(self):
-        return list(
-            get_dict_value_safe(
-                self.__group_operational_status, "available", {}
-            ).values()
-        )
+        data = self.__get_all_operational_statuses_from_operational_status()
+
+        if not data:
+            return None
+
+        return data.values()
 
     @property
     def available_operational_statuses_map(self):
-        return get_dict_value_safe(self.__group_operational_status, "available", {})
+        return self.__get_all_operational_statuses_from_operational_status()
+
+    @property
+    def operational_status_auxiliary_heater_3kw(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_POWER_STATUS", "COMP_VALUE_STEP_3KW"
+        )
+
+    @property
+    def operational_status_auxiliary_heater_6kw(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_POWER_STATUS", "COMP_VALUE_STEP_6KW"
+        )
+
+    @property
+    def operational_status_auxiliary_heater_9kw(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_POWER_STATUS", "COMP_VALUE_STEP_9KW"
+        )
+
+    @property
+    def operational_status_auxiliary_heater_12kw(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_POWER_STATUS", "COMP_VALUE_STEP_12KW"
+        )
+
+    @property
+    def operational_status_auxiliary_heater_15kw(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_POWER_STATUS", "COMP_VALUE_STEP_15KW"
+        )
+
+    @property
+    def operational_status_compressor_status(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_STATUS", "COMP_VALUE_COMPR"
+        )
+
+    @property
+    def operational_status_brine_pump_status(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_STATUS", "COMP_VALUE_BRINEPUMP"
+        )
+
+    @property
+    def operational_status_radiator_pump_status(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_STATUS", "COMP_VALUE_RADIATORPUMP"
+        )
+
+    @property
+    def operational_status_cooling_status(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_STATUS", "COMP_VALUE_COOLING"
+        )
+
+    @property
+    def operational_status_hot_water_status(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_STATUS", "COMP_VALUE_HOT_WATER"
+        )
+
+    @property
+    def operational_status_heating_status(self):
+        return self.__get_value_by_key_and_register_name_from_operational_status(
+            "COMP_STATUS", "COMP_VALUE_HEATING"
+        )
+
+    @property
+    def operational_status_integral(self):
+        data = self.__get_register_from_operational_status("REG_INTEGRAL_LSD")
+        return get_dict_value_safe(data, "registerValue")
 
     ###########################################################################
     # Operational time data
@@ -525,7 +686,7 @@ class ThermiaHeatPump:
         if self.__historical_data_registers_map is None:
             self.__set_historical_data_registers()
 
-        return list(self.__historical_data_registers_map.keys())
+        return list((self.__historical_data_registers_map or {}).keys())
 
     def get_historical_data_for_register(
         self, register_name, start_date: datetime, end_date: datetime
@@ -533,7 +694,9 @@ class ThermiaHeatPump:
         if self.__historical_data_registers_map is None:
             self.__set_historical_data_registers()
 
-        register_id = self.__historical_data_registers_map.get(register_name)
+        register_id = get_dict_value_safe(
+            self.__historical_data_registers_map, register_name
+        )
 
         if register_id is None:
             self._LOGGER.error("Register name is not supported: " + str(register_name))
@@ -601,7 +764,9 @@ class ThermiaHeatPump:
         print("self.__group_temperatures:")
         pretty_print_except(self.__group_temperatures)
 
-        installation_profile_id = self.__info.get("installationProfileId")
+        installation_profile_id = get_dict_value_safe(
+            self.__info, "installationProfileId"
+        )
 
         if installation_profile_id is not None:
             all_available_groups = self.__api_interface.get_all_available_groups(
